#ifndef __PACO_HPP__
#define __PACO_HPP__
#include <PSX/File/Processor/cd_file_processor.hpp>
#include <PSX/GPU/gpu.hpp>
#include <PSX/Timer/frame_timer.hpp>

namespace object {
    using namespace JabyEngine;

    class Paco {
    public:
        static constexpr auto TIM = SimpleTIM(896, 0, 960, 510);
    private:
        static const GPU::Color24 Colors[];

        GPU::TexPage::Linked tex_page;
        GPU::SPRT::Linked    sprite;
        SimpleTimer<uint8_t> timer;
        uint8_t              color_idx;

    public:
        constexpr Paco() : 
            tex_page(GPU::TexPage::create(GPU::PositionU16::create(
                TIM.get_texture_x(), TIM.get_texture_y()), 
                GPU::TexturePageColor::$4bit).linked()),
            sprite(GPU::SPRT::create(
                #pragma GCC warning "This pic used to be 122px (file size) and every tool would except it - however the display would be corrupt"
                GPU::AreaI16::create(GPU::PositionI16::create(0, 100), GPU::SizeI16::create(120, 128)),
                GPU::PagePositionClut(GPU::PagePosition::create(0, 0), GPU::PageClut::create(TIM.get_clut_x(), TIM.get_clut_y())),
                GPU::Color24::Blue()).linked()),
            timer(),
            color_idx(0) {}

            void setup();
            void update();
            void render();
    };
}

#endif //!__PACO_HPP__